@@ -210,7 +210,7 @@ HttpReply::packed304Reply()
 
 void
 HttpReply::setHeaders(HttpVersion ver, http_status status, const char *reason,
-                      const char *ctype, int64_t clen, time_t lmt, time_t expires)
+                      const char *ctype, int64_t clen, time_t lmt, time_t expiresTime)
 {
     HttpHeader *hdr;
     httpStatusLineSet(&sline, ver, status, reason);
@@ -228,8 +228,8 @@ HttpReply::setHeaders(HttpVersion ver, http_status status, const char *reason,
     if (clen >= 0)
         hdr->putInt64(HDR_CONTENT_LENGTH, clen);
 
-    if (expires >= 0)
-        hdr->putTime(HDR_EXPIRES, expires);
+    if (expiresTime >= 0)
+        hdr->putTime(HDR_EXPIRES, expiresTime);
 
     if (lmt > 0)		/* this used to be lmt != 0 @?@ */
         hdr->putTime(HDR_LAST_MODIFIED, lmt);
@@ -238,7 +238,7 @@ HttpReply::setHeaders(HttpVersion ver, http_status status, const char *reason,
 
     content_length = clen;
 
-    expires = expires;
+    expires = expiresTime;
 
     last_modified = lmt;
 }
@@ -458,24 +458,32 @@ HttpReply::sanityCheckStartLine(MemBuf *buf, const size_t hdr_len, http_status *
         return false;
     }
 
+    int pos;
     // catch missing or mismatched protocol identifier
-    if (protoPrefix.cmp(buf->content(), protoPrefix.size()) != 0) {
-        debugs(58, 3, "HttpReply::sanityCheckStartLine: missing protocol prefix (" << protoPrefix << ") in '" << buf->content() << "'");
-        *error = HTTP_INVALID_HEADER;
-        return false;
-    }
+    // allow special-case for ICY protocol (non-HTTP identifier) in response to faked HTTP request.
+    if (strncmp(buf->content(), "ICY", 3) == 0) {
+        protoPrefix = "ICY";
+        pos = protoPrefix.psize();
+    } else {
 
-    // catch missing or negative status value (negative '-' is not a digit)
-    int pos = protoPrefix.psize();
+        if (protoPrefix.cmp(buf->content(), protoPrefix.size()) != 0) {
+            debugs(58, 3, "HttpReply::sanityCheckStartLine: missing protocol prefix (" << protoPrefix << ") in '" << buf->content() << "'");
+            *error = HTTP_INVALID_HEADER;
+            return false;
+        }
 
-    // skip arbitrary number of digits and a dot in the verion portion
-    while ( pos <= buf->contentSize() && (*(buf->content()+pos) == '.' || xisdigit(*(buf->content()+pos)) ) ) ++pos;
+        // catch missing or negative status value (negative '-' is not a digit)
+        pos = protoPrefix.psize();
 
-    // catch missing version info
-    if (pos == protoPrefix.psize()) {
-        debugs(58, 3, "HttpReply::sanityCheckStartLine: missing protocol version numbers (ie. " << protoPrefix << "/1.0) in '" << buf->content() << "'");
-        *error = HTTP_INVALID_HEADER;
-        return false;
+        // skip arbitrary number of digits and a dot in the verion portion
+        while ( pos <= buf->contentSize() && (*(buf->content()+pos) == '.' || xisdigit(*(buf->content()+pos)) ) ) ++pos;
+
+        // catch missing version info
+        if (pos == protoPrefix.psize()) {
+            debugs(58, 3, "HttpReply::sanityCheckStartLine: missing protocol version numbers (ie. " << protoPrefix << "/1.0) in '" << buf->content() << "'");
+            *error = HTTP_INVALID_HEADER;
+            return false;
+        }
     }
 
     // skip arbitrary number of spaces...
