@@ -155,14 +155,14 @@ public:
         ~In();
         char *addressToReadInto() const;
 
-	ChunkedCodingParser *bodyParser; ///< parses chunked request body
-	MemBuf chunked; ///< contains unparsed raw (chunked) body data
-	MemBuf dechunked; ///< accumulates parsed (dechunked) content
+        ChunkedCodingParser *bodyParser; ///< parses chunked request body
+        MemBuf chunked; ///< contains unparsed raw (chunked) body data
+        MemBuf dechunked; ///< accumulates parsed (dechunked) content
         char *buf;
         size_t notYetUsed;
         size_t allocatedSize;
-	size_t chunkedSeen; ///< size of processed or ignored raw read data
-	DechunkingState dechunkingState; ///< request dechunking state
+        size_t chunkedSeen; ///< size of processed or ignored raw read data
+        DechunkingState dechunkingState; ///< request dechunking state
     } in;
 
     int64_t bodySizeLeft();
@@ -239,7 +239,7 @@ public:
      \param peer      if it is not NULL also check if the peer is the pinning peer
      \return          The fd of the server side connection or -1 if fails.
      */
-    int validatePinnedConnection(HttpRequest *request, const struct peer *peer=NULL);
+    int validatePinnedConnection(HttpRequest *request, const struct peer *peer);
     /**
      * returts the pinned peer if exists, NULL otherwise
      */
